@@ -17,17 +17,17 @@ order.
 =back
 
 Then, ack looks for a user-specific ackrc if the HOME environment
-variable is set.  This is either F<$HOME/.ackrc> or F<$HOME/_ackrc>.
+variable is set.  This is either `$HOME/.ackrc` or `$HOME/_ackrc`.
 
 Then, ack looks for a project-specific ackrc file.  ack searches
 up the directory hierarchy for the first `.ackrc` or `_ackrc` file.
 If this is one of the ackrc files found in the previous steps, it is
 not loaded again.
 
-It is a fatal error if a directory contains both F<.ackrc> and F<_ackrc>.
+It is a fatal error if a directory contains both `.ackrc` and `_ackrc`.
 
 After ack loads the options from the found ackrc files, ack looks
-at the C<ACKRC_OPTIONS> environment variable.
+at the ACKRC_OPTIONS environment variable.
 
 Finally, ack takes settings from the command line.
 
@@ -36,12 +36,11 @@ Finally, ack takes settings from the command line.
 use strict;
 use warnings;
 
-use App::Ack ();
 use App::Ack::ConfigDefault;
 use Cwd 3.00 ();
 use File::Spec 3.00;
 
-use if ($^O eq 'MSWin32'), 'Win32';
+use if ($^O =~ /MSWin32/ ? 1 : 0), "Win32";
 
 =head1 METHODS
 
@@ -51,29 +50,53 @@ Creates a new config finder.
 
 =cut
 
+our $is_win = 0;
+
 sub new {
     my ( $class ) = @_;
 
+    $is_win = $^O =~ /MSWin32/,
+
     return bless {}, $class;
 }
 
-
 sub _remove_redundancies {
-    my @configs = @_;
+    my ( @configs ) = @_;
 
-    my %seen;
-    foreach my $config (@configs) {
-        my $key = $config->{path};
-        if ( not $App::Ack::is_windows ) {
-            # On Unix, uniquify on inode.
-            my ($dev, $inode) = (stat $key)[0, 1];
-            $key = "$dev:$inode" if defined $dev;
+    if ( $is_win ) {
+        # inode stat always returns 0 on windows,
+        # so just check filenames
+        my (%seen, @uniq);
+
+        foreach my $path (@configs) {
+            push @uniq, $path unless $seen{$path};
+            $seen{$path} = 1;
         }
-        undef $config if $seen{$key}++;
-    }
-    return grep { defined } @configs;
-}
 
+        return @uniq;
+    }
+
+    else {
+
+        my %dev_and_inode_seen;
+
+        foreach my $path ( @configs ) {
+            my ( $dev, $inode ) = (stat $path)[0, 1];
+
+            if( defined($dev) ) {
+                if( $dev_and_inode_seen{"$dev:$inode"} ) {
+                    undef $path;
+                }
+                else {
+                    $dev_and_inode_seen{"$dev:$inode"} = 1;
+                }
+            }
+        }
+
+        return grep { defined() } @configs;
+
+    }
+}
 
 sub _check_for_ackrc {
     return unless defined $_[0];
@@ -89,56 +112,50 @@ sub _check_for_ackrc {
     return wantarray ? @files : $files[0];
 } # end _check_for_ackrc
 
-
 =head2 $finder->find_config_files
 
-Locates config files, and returns a list of them.
+Locates config files, and returns
+a list of them.
 
 =cut
 
 sub find_config_files {
     my @config_files;
 
-    if ( $App::Ack::is_windows ) {
-        push @config_files, map { +{ path => File::Spec->catfile($_, 'ackrc') } } (
+    if( $is_win ) {
+        push @config_files, map { File::Spec->catfile($_, 'ackrc') } (
             Win32::GetFolderPath(Win32::CSIDL_COMMON_APPDATA()),
             Win32::GetFolderPath(Win32::CSIDL_APPDATA()),
         );
     }
     else {
-        push @config_files, { path => '/etc/ackrc' };
+        push @config_files, '/etc/ackrc';
     }
 
 
     if ( $ENV{'ACKRC'} && -f $ENV{'ACKRC'} ) {
-        push @config_files, { path => $ENV{'ACKRC'} };
+        push @config_files, $ENV{'ACKRC'};
     }
     else {
-        push @config_files, map { +{ path => $_ } } _check_for_ackrc($ENV{'HOME'});
+        push @config_files, _check_for_ackrc($ENV{'HOME'});
     }
 
-    my $cwd = Cwd::getcwd();
-    return () unless defined $cwd;
-
-    # XXX This should go through some untainted cwd-fetching function, and not get untainted brute-force like this.
-    $cwd =~ /(.+)/;
-    $cwd = $1;
-    my @dirs = File::Spec->splitdir( $cwd );
+    my @dirs = File::Spec->splitdir(Cwd::getcwd());
     while(@dirs) {
         my $ackrc = _check_for_ackrc(@dirs);
         if(defined $ackrc) {
-            push @config_files, { project => 1, path => $ackrc };
+            push @config_files, $ackrc;
             last;
         }
         pop @dirs;
     }
 
-    # We only test for existence here, so if the file is deleted out from under us, this will fail later.
+    # XXX we only test for existence here, so if the file is
+    #     deleted out from under us, this will fail later. =(
     return _remove_redundancies( @config_files );
 }
 
-
-=head2 read_rcfile
+=head2 read_ackrc
 
 Reads the contents of the .ackrc file and returns the arguments.
 
@@ -158,11 +175,11 @@ sub read_rcfile {
         $line =~ s/\s+$//;
 
         next if $line eq '';
-        next if $line =~ /^\s*#/;
+        next if $line =~ /^#/;
 
         push( @lines, $line );
     }
-    close $fh or App::Ack::die( "Unable to close $file: $!" );
+    close $fh;
 
     return @lines;
 }
