@@ -4,10 +4,12 @@ use strict;
 use warnings;
 
 use App::Ack ();
+use App::Ack::ConfigDefault ();
+use App::Ack::ConfigFinder ();
 use App::Ack::Filter;
 use App::Ack::Filter::Default;
 use Carp 1.04 ();
-use Getopt::Long 2.36 ();
+use Getopt::Long 2.35 ();
 use Text::ParseWords 3.1 ();
 
 =head1 App::Ack::ConfigLoader
@@ -138,7 +140,9 @@ sub process_filetypes {
 
         if ( ref($args) ) {
             # $args are modified in place, so no need to munge $arg_sources
-            Getopt::Long::GetOptionsFromArray($args, %type_arg_specs);
+            local @ARGV = @{$args};
+            Getopt::Long::GetOptions(%type_arg_specs);
+            @{$args} = @ARGV;
         }
         else {
             ( undef, $arg_sources->[$i + 1] ) =
@@ -208,7 +212,7 @@ EOT
         'color-lineno=s'    => \$ENV{ACK_COLOR_LINENO},
         'column!'           => \$opt->{column},
         count               => \$opt->{count},
-        'create-ackrc'      => sub { App::Ack::create_ackrc(); exit; },
+        'create-ackrc'      => sub { print "$_\n" for ( '--ignore-ack-defaults', App::Ack::ConfigDefault::options() ); exit; },
         'env!'              => sub {
             my ( undef, $value ) = @_;
 
@@ -335,10 +339,11 @@ sub process_other {
 
     if ( $argv_source ) { # this *should* always be true, but you never know...
         my @copy = @{$argv_source};
+        local @ARGV = @copy;
 
         Getopt::Long::Configure('pass_through');
 
-        Getopt::Long::GetOptionsFromArray( \@copy,
+        Getopt::Long::GetOptions(
             'help-types' => \$is_help_types_active,
         );
 
@@ -352,7 +357,9 @@ sub process_other {
 
         my $ret;
         if ( ref($args) ) {
-            $ret = Getopt::Long::GetOptionsFromArray( $args, %{$arg_specs} );
+            local @ARGV = @{$args};
+            $ret = Getopt::Long::GetOptions( %{$arg_specs} );
+            @{$args} = @ARGV;
         }
         else {
             ( $ret, $arg_sources->[$i + 1] ) =
@@ -381,10 +388,12 @@ sub should_dump_options {
         my ( $name, $options ) = @{$sources}[$i, $i + 1];
         if($name eq 'ARGV') {
             my $dump;
+            local @ARGV = @{$options};
             Getopt::Long::Configure('default', 'pass_through', 'no_auto_help', 'no_auto_version');
-            Getopt::Long::GetOptionsFromArray($options,
+            Getopt::Long::GetOptions(
                 'dump' => \$dump,
             );
+            @{$options} = @ARGV;
             return $dump;
         }
     }
@@ -433,14 +442,14 @@ sub explode_sources {
             $j--;
 
             my @copy = @chunk;
-            Getopt::Long::GetOptionsFromArray(\@chunk,
+            local @ARGV = @chunk;
+            Getopt::Long::GetOptions(
                 'type-add=s' => $add_type,
                 'type-set=s' => $add_type,
                 'type-del=s' => $del_type,
             );
-            Getopt::Long::GetOptionsFromArray(\@chunk, %{$arg_spec});
+            Getopt::Long::GetOptions( %{$arg_spec} );
 
-            splice @copy, -1 * @chunk if @chunk; # XXX explain this
             push @new_sources, $name, \@copy;
         }
     }
@@ -517,9 +526,11 @@ sub remove_default_options_if_needed {
         my ( $name, $args ) = @{$sources}[ $index, $index + 1 ];
 
         if (ref($args)) {
-            Getopt::Long::GetOptionsFromArray($args,
+            local @ARGV = @{$args};
+            Getopt::Long::GetOptions(
                 'ignore-ack-defaults' => \$should_remove,
             );
+            @{$args} = @ARGV;
         }
         else {
             ( undef, $sources->[$index + 1] ) = Getopt::Long::GetOptionsFromString($args,
@@ -628,4 +639,56 @@ sub process_args {
     return \%opt;
 }
 
+
+sub retrieve_arg_sources {
+    my @arg_sources;
+
+    my $noenv;
+    my $ackrc;
+
+    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
+    Getopt::Long::Configure('pass_through');
+    Getopt::Long::Configure('no_auto_abbrev');
+
+    Getopt::Long::GetOptions(
+        'noenv'   => \$noenv,
+        'ackrc=s' => \$ackrc,
+    );
+
+    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
+
+    my @files;
+
+    if ( !$noenv ) {
+        my $finder = App::Ack::ConfigFinder->new;
+        @files  = $finder->find_config_files;
+    }
+    if ( $ackrc ) {
+        # we explicitly use open so we get a nice error message
+        # XXX this is a potential race condition!
+        if(open my $fh, '<', $ackrc) {
+            close $fh;
+        }
+        else {
+            die "Unable to load ackrc '$ackrc': $!"
+        }
+        push( @files, $ackrc );
+    }
+
+    push @arg_sources, Defaults => [ App::Ack::ConfigDefault::options() ];
+
+    foreach my $file ( @files) {
+        my @lines = App::Ack::ConfigFinder::read_rcfile($file);
+        push ( @arg_sources, $file, \@lines ) if @lines;
+    }
+
+    if ( $ENV{ACK_OPTIONS} && !$noenv ) {
+        push( @arg_sources, 'ACK_OPTIONS' => $ENV{ACK_OPTIONS} );
+    }
+
+    push( @arg_sources, 'ARGV' => [ @ARGV ] );
+
+    return @arg_sources;
+}
+
 1; # End of App::Ack::ConfigLoader
